social_media_replace_dict = {
    r"[\<\(\[]+(twitter|facebook|fb)[\]\)\>]+\(+.+\)+": "",
    r"[\<\(\[]+(twitter|facebook|fb)[\]\)\>]+": "",
}

# for confusing contractions, we simply took any value
contractions_replace_dict = {
    r"ain't": "are not",
    r"aren't": "are not",
    r"can't": "cannot",
    r"can't've": "cannot have",
    r"'cause": "because",
    r"could've": "could have",
    r"couldn't": "could not",
    r"couldn't've": "could not have",
    r"didn't": "did not",
    r"doesn't": "does not",
    r"don't": "do not",
    r"hadn't": "had not",
    r"hadn't've": "had not have",
    r"hasn't": "has not",
    r"haven't": "have not",
    r"he'd": "he would",
    r"he'd've": "he would have",
    r"he'll": "he will",
    r"he'll've": "he will have",
    r"he's": "he is",
    r"how'd": "how did",
    r"how'd'y": "how do you",
    r"how'll": "how will",
    r"how's": "how is",
    r"i'd": "I would",
    r"i'd've": "I would have",
    r"i'll": "I will",
    r"i'll've": "I will have",
    r"i'm": "I am",
    r"i've": "I have",
    r"isn't": "is not",
    r"it'd": "it would",
    r"it'd've": "it would have",
    r"it'll": "it will",
    r"it'll've": "it will have",
    r"it's": "it is",
    r"let's": "let us",
    r"ma'am": "madam",
    r"mayn't": "may not",
    r"might've": "might have",
    r"mightn't": "might not",
    r"mightn't've": "might not have",
    r"must've": "must have",
    r"mustn't": "must not",
    r"mustn't've": "must not have",
    r"needn't": "need not",
    r"needn't've": "need not have",
    r"o'clock": "of the clock",
    r"oughtn't": "ought not",
    r"oughtn't've": "ought not have",
    r"shan't": "shall not",
    r"sha'n't": "shall not",
    r"shan't've": "shall not have",
    r"she'd": "she would",
    r"she'd've": "she would have",
    r"she'll": "she will",
    r"she'll've": "she will have",
    r"she's": "she has / she is",
    r"should've": "should have",
    r"shouldn't": "should not",
    r"shouldn't've": "should not have",
    r"so've": "so have",
    r"so's": "so is",
    r"that'd": "that had",
    r"that'd've": "that would have",
    r"that's": "that is",
    r"there'd": "there would",
    r"there'd've": "there would have",
    r"there's": "there is",
    r"they'd": "they would",
    r"they'd've": "they would have",
    r"they'll": "they will",
    r"they'll've": "they will have",
    r"they're": "they are",
    r"they've": "they have",
    r"to've": "to have",
    r"wasn't": "was not",
    r"we'd": "we would",
    r"we'd've": "we would have",
    r"we'll": "we will",
    r"we'll've": "we will have",
    r"we're": "we are",
    r"we've": "we have",
    r"weren't": "were not",
    r"what'll": "what will",
    r"what'll've": "what will have",
    r"what're": "what are",
    r"what's": "what has / what is",
    r"what've": "what have",
    r"when's": "when is",
    r"when've": "when have",
    r"where'd": "where did",
    r"where's": "where is",
    r"where've": "where have",
    r"who'll": "who will",
    r"who'll've": "who will have",
    r"who's": "who is",
    r"who've": "who have",
    r"why's": "why is",
    r"why've": "why have",
    r"will've": "will have",
    r"won't": "will not",
    r"won't've": "will not have",
    r"would've": "would have",
    r"wouldn't": "would not",
    r"wouldn't've": "would not have",
    r"y'all": "you all",
    r"y'all'd": "you all would",
    r"y'all'd've": "you all would have",
    r"y'all're": "you all are",
    r"y'all've": "you all have",
    r"you'd": "you would",
    r"you'd've": "you would have",
    r"you'll": "you will",
    r"you'll've": "you will have",
    r"you're": "you are",
    r"you've": "you have",
    r'dm': 'direct message',
    r'thx': 'thanks',
    r'dming': 'direct messaging',
    r'dmed': 'direct messaged',
    r'plz': 'please',
    r'u': 'you',
    r'asap': 'as soon as possible',
    r'r': 'are',
}